/*
			Copyright (c) 1994 by
			Advanced Visual Systems Inc.
			All Rights Reserved
	
	This software comprises unpublished confidential information of
	Advanced Visual Systems Inc. and may not be used, copied or made
	available to anyone, except in accordance with the license
	under which it is furnished.
	
	This file is under Perforce control
	$Id: //depot/express/fcs992/include/avs/heap.h#1 $
*/
#ifndef HEAP_INCLUDED
#define HEAP_INCLUDED

#include <avs/dtx.h>

#ifdef __cplusplus
extern "C" {
#endif

typedef struct _HPheap {
    int heap_id;		/* the index of this heap in the file's heap
				   list.  This will be stored in the process
				   id structure for the object */
    int size_used;
    int size_alloced;
    char *heap_ptr;
    int vol_byte_size;
    int vol_num_ptrs; 
    int align;
    unsigned int fixed_size:1;
} HPheap;

typedef struct _HPheader {
   DTint size_used;
   DTint vol_byte_size;
   DTint vol_num_ptrs; 
} HPheader;

#define HPptr(H,O)  \
   (H->size_used > O ? H->heap_ptr + O : HPerror(__FILE__,__LINE__) )
    
#define HEAP_BLOCK_SIZE 16384

#define HPalign(nbytes,asize) \
	((nbytes) + ((asize - ((nbytes) % asize)) % asize))

/* volatile storage is always aligned to 8 byte boundaries */
#define HP_VOL_ALIGN	8

#define HPvcreate(SIZE)	HPcreate(SIZE,HP_VOL_ALIGN)

char   * HPerror           PTARGS((char *file, int line));
int      HPalloc           PTARGS((HPheap *heap, int size));
int      HPzero_alloc      PTARGS((HPheap *heap, int size));
char   * HPptr_alloc       PTARGS((HPheap *heap, int size));
char   * HPvptr_alloc       PTARGS((HPheap *heap, int size, 
				    int *alloced_in_heap));
int      HPvalloc          PTARGS((HPheap *heap, int size, 
                                   int vol_size, int vol_nptrs));
void     HPdestroy         PTARGS((HPheap *heap));
HPheap * HPcreate          PTARGS((int min_size, int align));
HPheap * HPmem_create      PTARGS((char *ptr, int align));
int      HPinsert_heap     PTARGS((HPheap *dst_heap, HPheap *src_heap));
HPheap * HPdefine_sub_heap PTARGS((HPheap *parent_heap, int offset));
int      HPwrite           PTARGS((HPheap *heap, FILE *fp));
HPheap * HPread            PTARGS((FILE *fp, int align));
HPheap * HPread_fd          PTARGS((int fd, int align));
int      HPwrite_c_heap    PTARGS((FILE *fp, char *name, HPheap *heap));
HPheap * HPdefine_heap     PTARGS((int size, int vol_byte_size, 
			           int vol_num_ptrs, int align, char *ptr));

#ifdef __cplusplus
} /* extern "C" */
#endif

#endif
